// This sample code tests the performance of sorting provided by
// "szsort_macros.h." Random 7-letters strings are sorted by
// HugeStringSort(), StringSort() and qsort().

#include "szsort_macros.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

// Buffer size (how many keys are in the buffer).
#define N (1 << 20)

// Key type.
typedef struct Key {
  char s[8];
} Key;

// Buffer to be sorted.
Key Buf[N];

// Function to get a letter from a Key.
static unsigned char GetLetter(const Key *key, size_t depth) {
  return (unsigned char)key->s[depth];
}

// Function to handle same keys found in sorting.
static void HandleSameKeys(Key *l, Key *r) {
}

// Define functions to sort an array of Keys.
DEFINE_SZSORT(StringSort, Key, GetLetter, HandleSameKeys);
DEFINE_HSZSORT(HugeStringSort, Key, GetLetter, HandleSameKeys);

// Timer.
static void Timer(int init) {
  static clock_t cl;
  if (init)
    cl = clock();
  else
    printf("%f sec\n", 1.0 * (clock() - cl) / CLOCKS_PER_SEC);
}

// Randomize a Key.
static void RandomizeKey(Key *key) {
  size_t i;
  for (i = 0; i < sizeof(key->s) - 1; i++)
    key->s[i] = '0' + rand() % 10;
  key->s[i] = '\0';
}

// Randomize Keys in buffer.
static void RandomizeBuffer(Key *buf, size_t n) {
  size_t i;
  for (i = 0; i < n; i++)
    RandomizeKey(buf + i);
}

// Confirm that an array is sorted.
static void IsSorted(const Key *key, size_t n) {
  size_t i;
  for (i = 1; i < n; i++)
    assert(strcmp(key[i - 1].s, key[i].s) <= 0);
}

int main(void) {
  Key *keys;

  srand((unsigned)time(NULL));

  RandomizeBuffer(Buf, N);

  // Test of HugeStringSort().
  Timer(1);
  keys = HugeStringSort(Buf, Buf + N, 0);
  if (keys != NULL) {
    Timer(0);
    IsSorted(keys, N);
    free(keys);
  }

  // Test of StringSort().
  Timer(1);
  StringSort(Buf, Buf + N, 0);
  Timer(0);
  IsSorted(Buf, N);

  return 0;
}
